/**
 * file: logic/openfoodfacts.go
 * author: theo technicguy
 * license: apache 2.0
 *
 * This file implements the connection logic to the
 * openfoodfacts service
 */

package logic

import (
	"fmt"
	"time"

	"git.licolas.net/hoffman/server/types"
	"github.com/openfoodfacts/openfoodfacts-go"
)

var (
	offAPI *openfoodfacts.Client
)

func setupOFFAPI() {
	offapi := openfoodfacts.NewClient("world", "", "")
	offAPI = &offapi
	offAPI.Timeout(5 * time.Second)
}

func getProductInformation(ean uint) (*types.Product, error) {
	if offAPI == nil {
		setupOFFAPI()
	}

	off, err := offAPI.Product(fmt.Sprintf("%d", ean))
	if err != nil {
		return nil, err
	}

	return &types.Product{
		Id:   0,
		EAN:  ean,
		Name: off.ProductName,
	}, nil
}

func fillBlankFields(product *types.Product) error {
	if product.EAN == 0 {
		return types.ErrInvalidSearch
	}

	off, err := getProductInformation(product.EAN)
	if err != nil {
		return err
	}

	if product.Name == "" {
		product.Name = off.Name
	}

	return nil
}
