/**
 * file: logic/product.go
 * author: theo technicguy
 * license: apache 2.0
 *
 * This file implements the product logic of the
 * logic provider
 */

package logic

import (
	"git.licolas.net/hoffman/server/types"
)

func (l *logic) checkProductExists(id uint) (*types.Product, error) {
	logger := logger.With().Uint("product-id", id).Logger()
	logger.Trace().Msg("checking product exists")
	product, err := l.dsrc.SelectProductById(id)
	if err != nil {
		return nil, err
	} else if product.Id == 0 {
		logger.Debug().Msg("product not found")
		return &types.Product{}, types.ErrIDNotFound
	}

	logger.Trace().Msg("product found")
	return product, nil
}

func (l *logic) CreateProduct(product *types.Product) (err error) {
	logger.Trace().Any("product", product).Msg("creating product")

	if err := fillBlankFields(product); err != nil {
		logger.Warn().Any("product", product).Err(err).Msg("error updating data")
	}

	return l.dsrc.InsertProduct(product)
}

func (l *logic) GetAllProducts() ([]*types.Product, error) {
	logger.Trace().Msg("getting all products")
	return l.dsrc.SelectAllProducts()
}

func (l *logic) GetProductByID(id uint) (product *types.Product, err error) {
	logger := logger.With().Uint("product-id", id).Logger()
	logger.Trace().Msg("getting product")

	product, err = l.checkProductExists(id)
	return
}

func (l *logic) GetProductsLike(search *types.ProductSearch) ([]*types.Product, error) {
	logger.Trace().Any("search", search).Msg("searching for product")
	return l.dsrc.SelectProductsLike(search)
}

func (l *logic) UpdateProduct(newProduct *types.Product) (*types.Product, error) {
	logger.Trace().Any("product", newProduct).Msg("updating product")
	if product, err := l.checkProductExists(newProduct.Id); err != nil {
		return product, err
	}

	return l.dsrc.UpdateProduct(newProduct)
}

func (l *logic) DeleteProduct(id uint) (*types.Product, error) {
	logger.Trace().Uint("product-id", id).Msg("deleting product")
	if product, err := l.checkProductExists(id); err != nil {
		return product, err
	}

	return l.dsrc.DeleteProduct(id)
}
