/**
 * file: logic/productTag.go
 * author: theo technicguy
 * license: apache 2.0
 *
 * This file implements the product tag logic of the
 * logic provider
 */

package logic

import "git.licolas.net/hoffman/server/types"

func (l *logic) checkTagExistsByID(id uint) (tag *types.Tag, err error) {
	logger := logger.With().Uint("tag-id", id).Logger()
	logger.Trace().Msg("checking tag exists")
	tag, err = l.dsrc.SelectProductTagById(id)
	if err != nil {
		return
	} else if tag.Id == 0 {
		logger.Debug().Msg("tag not found")
		return &types.Tag{}, types.ErrIDNotFound
	}

	logger.Trace().Msg("tag found")
	return
}

func (l *logic) checkTagExistsByName(name string) (tag *types.Tag, err error) {
	logger := logger.With().Str("tag-name", name).Logger()
	logger.Trace().Msg("checking tag exists")
	tag, err = l.dsrc.SelectProductTagByName(name)
	if err != nil {
		return
	} else if tag.Id == 0 {
		logger.Debug().Msg("tag not found")
		return &types.Tag{}, types.ErrIDNotFound
	}

	logger.Trace().Msg("tag found")
	return
}

func stripTag(_ *types.Tag, err error) error {
	logger.Trace().Msg("stripping tag")
	return err
}

func (l *logic) CreateProductTag(tag *types.Tag) error {
	logger.Trace().Any("tag", tag).Msg("creating tag")
	return l.dsrc.InsertProductTag(tag)
}

func (l *logic) GetAllProductTags() ([]*types.Tag, error) {
	logger.Trace().Msg("getting all tags")
	return l.dsrc.SelectAllProductTags()
}

func (l *logic) GetProductTagByID(id uint) (*types.Tag, error) {
	logger.Trace().Uint("tag-id", id).Msg("getting tag by id")
	return l.checkTagExistsByID(id)
}

func (l *logic) GetProductTagByName(name string) (*types.Tag, error) {
	logger.Trace().Str("tag-name", name).Msg("getting tag by name")
	return l.checkTagExistsByName(name)
}

func (l *logic) GetProductTagsLikeName(name string) ([]*types.Tag, error) {
	logger.Trace().Str("tag-name", name).Msg("getting tag like name")
	return l.dsrc.SelectProductTagsLikeName(name)
}

func (l *logic) UpdateProductTag(tag *types.Tag) (*types.Tag, error) {
	logger.Trace().Any("tag", tag).Msg("updating tag")
	if tag, err := l.checkTagExistsByID(tag.Id); err != nil {
		return tag, err
	}

	return l.dsrc.UpdateProductTag(tag)
}

func (l *logic) DeleteProductTag(id uint) (*types.Tag, error) {
	logger.Trace().Uint("tag-id", id).Msg("deleting tag")
	if err := stripTag(l.checkTagExistsByID(id)); err != nil {
		return nil, err
	}

	return l.dsrc.DeleteProductTag(id)
}
