/**
 * file: tests/api/v0/mockLogicProvider_test.go
 * author: Theo Technicguy
 * license: Apache 2.0
 *
 * API tests utility functions
 */

package v0_test

import (
	"errors"

	"git.licolas.net/hoffman/server/types"
	"github.com/stretchr/testify/mock"
)

var (
	ErrConversion = errors.New("mock conversion error")
)

type mockedLogic struct {
	mock.Mock
}

func (m *mockedLogic) CreateProduct(product *types.Product) error {
	args := m.Called(product)
	return args.Error(0)
}

func (m *mockedLogic) GetAllProducts() ([]*types.Product, error) {
	args := m.Called()

	switch v := args.Get(0).(type) {
	case []*types.Product:
		return v, args.Error(1)
	case nil:
		return nil, args.Error(1)
	default:
		return nil, ErrConversion
	}
}

func (m *mockedLogic) GetProductByID(id uint) (*types.Product, error) {
	args := m.Called(id)

	switch v := args.Get(0).(type) {
	case *types.Product:
		return v, args.Error(1)
	case nil:
		return nil, args.Error(1)
	default:
		return nil, ErrConversion
	}
}

func (m *mockedLogic) GetProductsLike(search *types.ProductSearch) ([]*types.Product, error) {
	args := m.Called(search)

	switch v := args.Get(0).(type) {
	case []*types.Product:
		return v, args.Error(1)
	case nil:
		return nil, args.Error(1)
	default:
		return nil, ErrConversion
	}
}

func (m *mockedLogic) UpdateProduct(product *types.Product) (*types.Product, error) {
	args := m.Called(product)

	switch v := args.Get(0).(type) {
	case *types.Product:
		return v, args.Error(1)
	case nil:
		return nil, args.Error(1)
	default:
		return nil, ErrConversion
	}
}

func (m *mockedLogic) DeleteProduct(id uint) (*types.Product, error) {
	args := m.Called(id)

	switch v := args.Get(0).(type) {
	case *types.Product:
		return v, args.Error(1)
	case nil:
		return nil, args.Error(1)
	default:
		return nil, ErrConversion
	}
}

func (m *mockedLogic) CreateProductTag(tag *types.Tag) error {
	args := m.Called(tag)
	return args.Error(0)
}

func (m *mockedLogic) GetAllProductTags() ([]*types.Tag, error) {
	args := m.Called()

	switch v := args.Get(0).(type) {
	case []*types.Tag:
		return v, args.Error(1)
	case nil:
		return nil, args.Error(1)
	default:
		return nil, ErrConversion
	}
}

func (m *mockedLogic) GetProductTagByID(id uint) (*types.Tag, error) {
	args := m.Called(id)

	switch v := args.Get(0).(type) {
	case *types.Tag:
		return v, args.Error(1)
	case nil:
		return nil, args.Error(1)
	default:
		return nil, ErrConversion
	}
}

func (m *mockedLogic) GetProductTagByName(name string) (*types.Tag, error) {
	args := m.Called(name)

	switch v := args.Get(0).(type) {
	case *types.Tag:
		return v, args.Error(1)
	case nil:
		return nil, args.Error(1)
	default:
		return nil, ErrConversion
	}
}

func (m *mockedLogic) GetProductTagsLikeName(name string) ([]*types.Tag, error) {
	args := m.Called(name)

	switch v := args.Get(0).(type) {
	case []*types.Tag:
		return v, args.Error(1)
	case nil:
		return nil, args.Error(1)
	default:
		return nil, ErrConversion
	}
}

func (m *mockedLogic) UpdateProductTag(tag *types.Tag) (*types.Tag, error) {
	args := m.Called(tag)

	switch v := args.Get(0).(type) {
	case *types.Tag:
		return v, args.Error(1)
	case nil:
		return nil, args.Error(1)
	default:
		return nil, ErrConversion
	}
}

func (m *mockedLogic) DeleteProductTag(id uint) (*types.Tag, error) {
	args := m.Called(id)

	switch v := args.Get(0).(type) {
	case *types.Tag:
		return v, args.Error(1)
	case nil:
		return nil, args.Error(1)
	default:
		return nil, ErrConversion
	}
}
