/**
 * file: tests/database/connector_test.go
 * author: Theo Technicguy
 * license: Apache 2.0
 *
 * Database connector tests
 */

package database_test

import (
	"database/sql"
	"os"
	"testing"

	"git.licolas.net/hoffman/server/database"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func testDatabaseStructure(t *testing.T, db *sql.DB, expect []string) {
	rows, err := db.Query(`SELECT tbl_name
	FROM sqlite_master
	WHERE type = 'table' AND NOT tbl_name LIKE 'sqlite%%';
	`)
	require.NoError(t, err)

	actual := []string{}
	for rows.Next() {
		var table string
		err := rows.Scan(&table)
		require.NoError(t, err)
		actual = append(actual, table)
	}

	assert.ElementsMatch(t, actual, expect)
}

// TestConnect tests if opening a new database creates the required tables
func TestConnect(t *testing.T) {
	fp := tmpPath()
	db := setupTest(t, fp)
	dsrc, err := database.Connect(fp)
	require.NoError(t, err)
	assert.NotNil(t, dsrc)

	defer teardownTest(t, dsrc, db, fp)

	expect := []string{"Product", "ProductTag", "ProductTagging"}
	testDatabaseStructure(t, db, expect)
}

func TestConnectNoCreate(t *testing.T) {
	fp := tmpPath()
	db := setupTest(t, fp)
	_, err := db.Exec(`CREATE TABLE testtable (id INTEGER, val VARCHAR(64));`)
	require.NoError(t, err)

	dsrc, err := database.Connect(fp)
	require.NoError(t, err)
	defer teardownTest(t, dsrc, db, fp)

	expect := []string{"testtable"}
	testDatabaseStructure(t, db, expect)
}

func TestDisconnect(t *testing.T) {
	fp := tmpPath()
	dsrc, err := database.Connect(fp)
	require.NoError(t, err)
	defer os.Remove(fp)

	err = dsrc.Disconnect()
	require.NoError(t, err)
}
