/**
 * file: tests/database/productTagging_test.go
 * author: Theo Technicguy
 * license: Apache 2.0
 *
 * Database product connector tests
 */

package database_test

import (
	"database/sql"
	"testing"

	"git.licolas.net/hoffman/server/types"
	"github.com/stretchr/testify/require"
)

func testCreateProductTaggingTable(t *testing.T, db *sql.DB) {
	_, err := db.Exec(`CREATE TABLE ProductTagging (
		ProductID INTEGER NOT NULL,
		TagID INTEGER NOT NULL,
		PRIMARY KEY (ProductID, TagID),
		CONSTRAINT productId FOREIGN KEY (ProductID)
			REFERENCES Product (ProductID)
			ON UPDATE CASCADE
			ON DELETE RESTRICT,
		CONSTRAINT tagId FOREIGN KEY (TagID)
			REFERENCES ProductTag (TagID)
			ON UPDATE CASCADE
			ON DELETE RESTRICT
	);`)
	require.NoError(t, err)
}

func testInsertProductTagging(t *testing.T, db *sql.DB, pid, tid uint) {
	query := "INSERT INTO ProductTagging (ProductID, TagID) VALUES (?, ?)"
	args := []any{pid, tid}
	testExec(t, db, query, args...)
}

func testSelectTaggingsByProductID(t *testing.T, db *sql.DB, pid uint) []types.Tag {
	query := `SELECT pt.TagID, pt.Name, pt.Color
	FROM ProductTag pt
	JOIN ProductTagging ptg on pt.TagID = ptg.TagID
	WHERE ProductID = ?;`
	scanner := func(rows *sql.Rows) (tag types.Tag, err error) {
		tag = types.Tag{}
		err = rows.Scan(&tag.Id, &tag.Name, &tag.Color)
		return
	}
	return testSelect[types.Tag](t, db, scanner, query, pid)
}
